import fs from "fs";
import path from "upath";
import commentJSON from "comment-json";
import { plugin as workspacesPlugin } from "./manager/workspaces";
import assert from "assert";
export const DEFAULT_TSCONFIGPATH = "tsconfig.json";
const sortReferences = (references) => {
    return references.slice().sort((refA, refB) => (refA.path > refB.path ? 1 : -1));
};
export const toProjectReferences = (options) => {
    const plugins = Array.isArray(options.plugins) && options.plugins.length > 0 ? options.plugins : [workspacesPlugin];
    const pluginImplementations = plugins.map((plugin) => plugin(options));
    // use first plugin
    const supportPlugin = pluginImplementations.find((plugin) => {
        return plugin.supports();
    });
    if (!supportPlugin) {
        throw new Error("Not found supported plugin");
    }
    const relativeName = (pkgPath) => {
        return path.relative(options.rootDir, pkgPath);
    };
    const allPackages = supportPlugin.getAllPackages();
    const errors = [];
    allPackages.forEach((packageInfo) => {
        var _a, _b, _c;
        const tsconfigFilePath = (_b = (_a = options.tsConfigPathFinder) === null || _a === void 0 ? void 0 : _a.call(options, packageInfo.location)) !== null && _b !== void 0 ? _b : path.join(packageInfo.location, DEFAULT_TSCONFIGPATH);
        if (!fs.existsSync(tsconfigFilePath)) {
            // Skip has not tsconfig.json
            return;
        }
        const tsconfigJSON = commentJSON.parse(fs.readFileSync(tsconfigFilePath, "utf-8"));
        const references = supportPlugin.getDependencies(packageInfo.packageJSON);
        const newProjectReferences = references
            .map((reference) => {
            var _a;
            const absolutePathOrNull = supportPlugin.resolve(reference);
            if (!absolutePathOrNull) {
                return;
            }
            const tsConfigDirName = path.dirname((_a = options.tsConfigPath) !== null && _a !== void 0 ? _a : DEFAULT_TSCONFIGPATH);
            const absolutePath = path.join(absolutePathOrNull, tsConfigDirName);
            if (!path.isAbsolute(absolutePath)) {
                throw new Error(`Plugin#resolve should return absolute path: ${absolutePath}, plugin: ${supportPlugin}`);
            }
            if (packageInfo.location === absolutePath) {
                const selfName = relativeName(packageInfo.location);
                errors.push(new Error(`[${selfName}] Self dependencies is something wrong: ${selfName} refer to ${relativeName(absolutePath)}`));
            }
            const resolvePath = path.dirname(path.resolve(packageInfo.location, tsconfigFilePath));
            return {
                path: path.relative(resolvePath, absolutePath)
            };
        })
            .filter((r) => Boolean(r));
        const currentProjectReferences = (_c = tsconfigJSON["references"]) !== null && _c !== void 0 ? _c : [];
        if (options.checkOnly) {
            // check
            try {
                // create pure object for compare
                const cleanCurrentProjectReferences = JSON.parse(JSON.stringify(commentJSON.parse(commentJSON.stringify(currentProjectReferences), undefined, true)));
                // TODO: move sorting to updating logic when next major release
                // https://github.com/azu/monorepo-utils/issues/44
                assert.deepStrictEqual(sortReferences(cleanCurrentProjectReferences), sortReferences(newProjectReferences));
            }
            catch (error) {
                const selfName = relativeName(packageInfo.location);
                errors.push(new Error(`[${selfName}] ${error.message}`));
            }
            return;
        }
        else {
            // update
            if (errors.length === 0) {
                tsconfigJSON["references"] = newProjectReferences;
                const contents = commentJSON.stringify(tsconfigJSON, null, 2);
                fs.writeFileSync(tsconfigFilePath, `${contents}\n`, "utf-8");
            }
        }
    });
    if (errors.length > 0) {
        return {
            ok: false,
            aggregateError: {
                message: `workspaces-to-typescript-project-references found ${errors.length} errors.

Please update your tsconfig.json via following command.

$ workspaces-to-typescript-project-references
`,
                errors
            }
        };
    }
    return {
        ok: true
    };
};
//# sourceMappingURL=index.js.map